using System;
using System.Text.RegularExpressions;

namespace sharpMail
{
	/// <summary>
	/// Classe per la gestione del protocollo pop3
	/// </summary>
	public class pop
	{

		/// <summary>
		/// Socket stream per leggere i dati dal socketTcp
		/// </summary>
		private System.Net.Sockets.NetworkStream sckStreamSocket;
		///<summary>
		/// Socket tcp ip
		/// </summary>
		private System.Net.Sockets.TcpClient sckTcpClient = new System.Net.Sockets.TcpClient();
		/// <summary>
		/// Indicatore di connessione
		/// </summary>
		private Boolean blnConnect = false;
		/// <summary>
		/// Numero dei messaggi in casella
		/// </summary>
		private int _intMessageCount;
		/// <summary>
		/// Dimensione della casella
		/// </summary>
		private int _intMailboxSize;
		/// <summary>
		/// Contenuto del messaggio di posta
		/// </summary>
		private string _strMessage;
		/// <summary>
		/// Stringa messaggio del server
		/// </summary>
		private string cmdString;
		/// <summary>
		/// Destinatario
		/// </summary>
		private string _strTo; 
		/// <summary>
		/// Mittente
		/// </summary>
		private string _strFrom;
		/// <summary>
		/// Destinatari
		/// </summary>
		private string _strCC;
		/// <summary>
		/// Oggetto email
		/// </summary>
		private string _strSubject;

		/// <summary>
		/// Destinatario
		/// </summary>
		public string strTo
		{
			get
			{
				return _strTo;
			}
		} 
		
		/// <summary>
		/// Mittente
		/// </summary>
		public string strFrom
		{
			get
			{
				return _strFrom;
			}
		} 
		
		/// <summary>
		/// Destinatari
		/// </summary>
		public string strCC
		{
			get
			{
				return _strCC;
			}
		}
		
		/// <summary>
		/// Oggetto mail
		/// </summary>
		public string strSubject
		{
			get
			{
				return _strSubject;
			}
		}

		/// <summary>
		/// Testo messaggio di posta
		/// </summary>
		public string strMessage
		{
			get
			{
				return _strMessage;
			}	   
		}
												  
		/// <summary>
		/// Numero dei messaggi in posta
		/// </summary>
		public int intMessageCount
		{
			get
			{
				return _intMessageCount;
			}
		}
													  
		/// <summary>
		/// Dimensione della casella di posta
		/// </summary>
		public int intMailboxSize
		{
			get
			{
				return _intMailboxSize;
			}
		}
		
		/// <summary>
		/// Messaggio che ritorna il server dopo un l'esecuzione di un comando
		/// </summary>
		public string messageString
		{
			get
			{
				return cmdString;
			}
		}
		
		/// <summary>
		/// Propriet che controlla la connessione
		/// </summary>
		/// <returns>True se connsessi, false se non connessi</returns>
		public bool isConnect
		{
			get
			{
				return blnConnect;
			}
		}

		/// <summary>
		/// Costruttor
		/// </summary>
		public pop()
		{
		}

		/// <summary>
		/// Metodo che chiude la connessione
		/// </summary>
		public void closeConnection()
		{
			//controllo che sia connesso
			if(blnConnect)
			{
				sckTcpClient.Close();
				sckStreamSocket.Close();
			}
		}
					   
		/// <summary>
		/// Metodo che connette al server pop3
		/// </summary>
		/// <param name="Server">Indirizzo del server di posta</param>
		public void connect(string Server)
		{
			//apro la porta 110 del mio server di posta
			try
			{
				//Provo a connettermi
				sckTcpClient.Connect(Server, 110);
				//passo al socket stream i valori dello stream tcp
				sckStreamSocket = sckTcpClient.GetStream();
				//controllo che la connessione sia avvenuta correttamente
				//per far ci controllo che i primi 3 caratteri siano +OK
				if(chekMessage(getServerData()))
				{
					blnConnect = true;
				}
			}
			catch(Exception ex)
			{
			}
		}
		
		/// <summary>
		/// Metodo che ritorna il messaggio del server di posta
		/// </summary>
		/// <returns>Stringa con il messaggio del server</returns>
		private string getServerData()
		{
			string strServerData;
			//dichiaro la matrice di byte
			byte[] byteServer = new byte[sckTcpClient.ReceiveBufferSize];
			strServerData = "";
			do
			{
				//leggo l'array di byte che mi restituisce il server
				sckStreamSocket.Read(byteServer, 0, byteServer.Length);
				//concateno il messaggio
				strServerData += System.Text.Encoding.ASCII.GetString(byteServer);
				//Pulisco l'array di byte
				//byteServer.Clear(byteServer, 0, byteServer.Length);
				System.Threading.Thread.Sleep(250);
			}
			//Ciclo fino a quando ci sono dati leggibili nell'array di byte
			while(sckStreamSocket.DataAvailable);
			return strServerData;
		}
											 
		/// Metodo che esegue un comando
		/// </summary>
		/// <param name="Cmd">Testo del comando</param>
		/// <returns>Messaggio del server</returns>
		private string executeCommand(string Cmd)
		{
			byte[] byteCommand;
			//Codifico il comando in byte
			byteCommand = System.Text.Encoding.ASCII.GetBytes(Cmd + "\n");
			//Invio il comando al server
			sckStreamSocket.Write(byteCommand, 0, byteCommand.Length);
			//ricavo i dati di risposta dal server
			return getServerData();
		}
		
		/// <summary>
		/// Metodo che controlla il messaggio del server dopo una operazione
		/// </summary>
		/// <param name="strMessage">Messaggio del server</param>
		/// <returns>
		/// True se l'operazione  stata eseguita con successo,
		/// false errore
		/// </returns>
		private bool chekMessage(string strMessage)
		{
			//controllo le prime tre letter del messaggio
			//se con +OK l'operazione  stata eseguita con successo
			if(strMessage.StartsWith("+OK"))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		
		/// <summary>
		/// Metodo per la disconnessione dal server
		/// </summary>
		/// <returns>True se l'operazione  stata eseguita correttamente, false comando rifiutato</returns>
		public bool logOut()
		{
			//controllo che sia connesso
			if(blnConnect)
			{
				cmdString = executeCommand("QUIT");
				if(chekMessage(cmdString))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}
																																			 
		/// <summary>
		/// Metodo che elimina un messaggio
		/// </summary>
		/// <param name="intMessageNum">Numero del messaggio</param>
		/// <returns>True se l'operazione  stata eseguita correttamente, false comando rifiutato</returns>
		public bool deleteMessage(int intMessageNum)
		{
			//controllo che sia connesso
			if(blnConnect)
			{
				cmdString = executeCommand("DELE " + intMessageNum.ToString());
				if(chekMessage(cmdString))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}
		
		/// <summary>
		/// Metodo che restituisc il testo di un messaggio
		/// </summary>
		/// <param name="intMessageNum">Numero id del messaggio</param>
		public void getMessage(int intMessageNum)
		{
			string strHeader;
			int intHeaderEnd;
			//controllo che sia connesso
			if(blnConnect)
			{
				_strMessage = executeCommand("RETR " + intMessageNum.ToString());
				//controllo che il messaggio sia stato ricevuto
				if(chekMessage(_strMessage))
				{
					//cerco la fine degli header, e la fine  caratterizzata dalla prima
					//riga dopo due a capi
					strHeader = executeCommand("TOP " + intMessageNum.ToString());
					//imposto gli header
					getHeader(strHeader);
					//reimposto il testo del messaggio
					_strMessage = _strMessage.Replace(strHeader, "");
				}
			}
		}

		/// <summary>
		/// Metodo che imposta gli header del messaggio
		/// </summary>
		/// <param name="intMessageNum">Numero del messaggio</param>
		public void getMailHeader(int intMessageNum){
			string strHeader;
			if(blnConnect){
				strHeader = executeCommand("TOP " + intMessageNum.ToString());
				if(chekMessage(strHeader)){
					getHeader(strHeader);
				}
			}
		}
							 
		/// <summary>
		/// Metodo che ricava le propriet dagli header del messaggio
		/// </summary>
		/// <param name="strHeader">Header del messaggio</param>
		private void getHeader(string strHeader)
		{
			//estraggo le principali propriet di un messaggio tramite espressioni regolari
			_strTo = Regex.Match(strHeader, "\nTo:(?<to>[^\n]+)\n", RegexOptions.IgnoreCase).Value.ToString();
			_strFrom = Regex.Match(strHeader, "\nFrom:(?<from>[^\n]+)\n", RegexOptions.IgnoreCase).Value.ToString();
			_strCC = Regex.Match(strHeader, "\nCC:(?<cc>[^\n]+)\n", RegexOptions.IgnoreCase).Value.ToString();
			_strSubject = Regex.Match(strHeader, "\nSubject:(?<subject>[^\n]+)\n", RegexOptions.IgnoreCase).Value.ToString();
		}

		/// <summary>
		/// Metodo che ricava il numero dei messaggi e lo spazio occupato
		/// </summary>
		public void mailboxStats()
		{
			char[] splitter  = {' '};
			if(blnConnect)
			{
				//dichiaro un array di stringhe
				string[] arrayStat;
				//invio il comando
				arrayStat = executeCommand("STAT").Split(splitter);
				if(chekMessage(arrayStat[0]))
				{
					//ricavo il numero di messaggi
					_intMessageCount = Convert.ToInt32(arrayStat[1].ToString());
					//ricavo la dimendione della casella di posta
					_intMailboxSize = Convert.ToInt32(arrayStat[2].ToString());
				}
			}
		}

		/// <summary>
		/// Metodo che mi ritorna la dimensione di un messaggio di posta
		/// </summary>
		/// <param name="intMessageNum">Numero id del messaggio</param>
		/// <returns>Dimensione del messaggio</returns>
		public int messageSize(int intMessageNum)
		{
			char[] splitter  = {' '};
			//controllo che sia connesso
			if(blnConnect)
			{
				//dichiaro un array di stringhe
				string[] arrayStat;
				//invio il comando
				arrayStat = executeCommand("LIST " + intMessageNum.ToString()).Split(splitter);
				if(chekMessage(arrayStat[0]))
				{
					//ricavo la dimendione della casella di posta
					return Convert.ToInt32(arrayStat[2].ToString());
				}
				else{
					return -1;
				}
			}
			else
			{
				return -1;
			}
		}

		
		
		/// <summary>
		/// Metodo per il login al server
		/// </summary>
		/// <param name="userName">Nome utente</param>
		/// <param name="passWord">Password</param>
		/// <returns>True se si  loggati con successo, false utente rifiutato</returns>
		public bool login(string userName, string passWord){
			//controllo di essere collegato al server
			if(blnConnect){
				//Invio la userName
				cmdString = executeCommand("USER " + userName);
				//controllo il messaggio del server
				if(chekMessage(cmdString))
				{
					cmdString = executeCommand("PASS " + passWord);
					//controllo che la password sia esatta
					if(chekMessage(cmdString))
					{
						return true;
					}
					else
					{
						return false;
					}
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}
		
	}
}
